//region Imports
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
var git = require('gulp-git');
var deploy = require('gulp-gh-pages');
var fs = require('fs');
var cwd = require('cwd');
var confirm = require('inquirer-confirm');

var _require = require('child_process');

var spawn = _require.spawn;
var execSync = _require.execSync;

var _ = require('lodash');
//endregion

exports['default'] = function (gulp, opts) {

  gulp.task('publish:confirm', function (done) {
    var version = execSync('npm show . version', { encoding: 'utf8' });
    var pjson = require(cwd('package.json'));
    console.log('Last published version:', _.trim(version));
    console.log('Local package version:', pjson.version);
    confirm('Are you sure you want to publish/tag v' + pjson.version).then(function () {
      done();
    }, done);
  });

  gulp.task('publish:tag', function (done) {
    var pjson = require(cwd('package.json'));
    var v = 'v' + pjson.version;
    var message = 'Release ' + v;
    git.tag(v, message, function (e) {
      if (e) throw e;
      git.push('origin', v, function (e) {
        if (e) throw e;
        done();
      });
    });
  });

  gulp.task('publish:npm', function (done) {
    var pkg = require(cwd('package.json'));
    confirm('Are you sure you want to publish v' + pkg.version).then(function () {
      spawn('npm', ['publish'], { stdio: 'inherit' }).on('close', done);
    }, done);
  });

  gulp.task('release', gulp.series('publish:npm', 'publish:tag', 'publish:confirm'));
};

module.exports = exports['default'];