//region Imports
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
var mocha = require('gulp-mocha');
var jeditor = require('gulp-json-editor');
var _ = require('lodash');
var karma = require('karma');
var cwd = require('cwd');
var conflict = require('gulp-conflict');
var modifyPackageJson = require('../../util/modifyPackageJson');
var path = require('path');
//endregion

exports['default'] = function (gulp, opts) {

  _.defaults(opts, {
    srcGlob: '',
    testsGlob: '',
    mocha: {
      require: ['babel/register', './src/__tests__/bootstrap.js']
    },
    //reporter: 'nyan',
    testsDestGlob: ''
  });

  // TODO(vjpr): Not teted.
  gulp.task('karma:start', function (done) {
    new karma.Server({
      configFile: cwd('karma.conf.js'),
      singleRun: true
    }, done).start();
  });

  gulp.task('mocha:src', function () {
    return gulp.src(opts.testsGlob, { read: false }).pipe(mocha(opts.mocha));
  });

  // Faster: Run tests on transpiled files so no need for slow babel/require.
  gulp.task('mocha:lib:run', function () {

    if (!opts.shouldCompileTests) throw 'shouldCompileTests must be enabled for test-transpiled task';

    // TODO(vjpr): Should we clone first?
    opts.mocha.require = _.without(opts.mocha.require, 'babel/register');
    return gulp.src(opts.testsDestGlob, { read: false }).pipe(mocha(opts.mocha));
  });

  gulp.task('mocha:lib', gulp.series('lib:compile', 'mocha:lib:run'));

  gulp.task('test:watch', gulp.series('mocha:lib', function watcher() {
    gulp.isWatching = true;
    return gulp.watch(opts.srcGlob, gulp.series('mocha:lib')).on('change', function (evt) {
      gutil.log('---');
      gutil.log('File changed:', gutil.colors.red(evt.path));
    });
  }));

  gulp.task('test:bootstrap:pjson', function (done) {
    modifyPackageJson(function (json) {
      _.set(json, 'scripts.karma', './node_modules/karma/bin/karma start');
      _.set(json, 'scripts.test', 'gulp test');
      return json;
    }, done);
  });

  gulp.task('test:bootstrap:copy-files', function () {
    // TODO(vjpr): This is hacky. We need to src the precompiled version.
    // Move to templates dir or ignore when compiling.
    return gulp.src(path.join(opts.srcDir, 'test/template/**/*.*')).pipe(conflict('.')).pipe(gulp.dest(opts.destDir));
  });

  gulp.task('test:bootstrap', gulp.series('test:bootstrap:copy-files', 'test:bootstrap:pjson'));
};

module.exports = exports['default'];